@@ -6,6 +6,7 @@ import inspect
 import logging
 import platform
 import os
+import subprocess
 import sys
 
 from six.moves import input
@@ -391,6 +392,13 @@ class Command(object):
                    metavar='FILENAME',
                    help='Uploads an existing diff file, instead of '
                         'generating a new diff.'),
+        ]
+    )
+
+    branch_options = OptionGroup(
+        name='Branch Options',
+        description='Options for selecting branches.',
+        option_list=[
             Option('--tracking-branch',
                    dest='tracking',
                    metavar='BRANCH',
@@ -459,6 +467,7 @@ class Command(object):
                    help='The password for the SVN repository.'),
             Option('--svn-prompt-password',
                    dest='svn_prompt_password',
+                   config_key='SVN_PROMPT_PASSWORD',
                    default=False,
                    action='store_true',
                    help="Prompt for the user's svn password. This option "
@@ -496,6 +505,21 @@ class Command(object):
                         'user@domain,password). Visit https://msdn.microsoft.'
                         'com/en-us/library/hh190725.aspx to learn about '
                         'saving credentials for reuse.'),
+            Option('--tf-cmd',
+                   dest='tf_cmd',
+                   default=None,
+                   metavar='TF_CMD',
+                   config_key='TF_CMD',
+                   help='The full path of where to find the tf command. This '
+                        'overrides any detected path.'),
+            Option('--tfs-shelveset-owner',
+                   dest='tfs_shelveset_owner',
+                   default=None,
+                   metavar='TFS_SHELVESET_OWNER',
+                   help='When posting a shelveset name created by another '
+                        'user (other than the one who owns the current '
+                        'workdir), look for that shelveset using this '
+                        'username.'),
         ]
     )
 
@@ -584,6 +608,23 @@ class Command(object):
         logging.debug('Home = %s', get_home_path())
         logging.debug('Current directory = %s', os.getcwd())
 
+    def create_arg_parser(self, argv):
+        """Create and return the argument parser.
+
+        Args:
+            argv (list of unicode):
+                A list of command line arguments
+
+        Returns:
+            argparse.ArgumentParser:
+            Argument parser for commandline arguments
+        """
+        self.config = load_config()
+        parser = self.create_parser(self.config, argv)
+        parser.add_argument('args', nargs=argparse.REMAINDER)
+
+        return parser
+
     def run_from_argv(self, argv):
         """Execute the command using the provided arguments.
 
@@ -591,12 +632,9 @@ class Command(object):
         from ``argv`` and the commands ``main`` method will
         be called.
         """
-        self.config = load_config()
-
-        parser = self.create_parser(self.config, argv)
-        parser.add_argument('args', nargs=argparse.REMAINDER)
-
+        parser = self.create_arg_parser(argv)
         self.options = parser.parse_args(argv[2:])
+
         args = self.options.args
 
         # Check that the proper number of arguments have been provided.
@@ -617,6 +655,7 @@ class Command(object):
             sys.exit(1)
 
         self.init_logging()
+        logging.debug('Command line: %s', subprocess.list2cmdline(argv))
 
         try:
             exit_code = self.main(*args) or 0
@@ -667,11 +706,25 @@ class Command(object):
         tool.capabilities = self.get_capabilities(api_root)
 
     def get_server_url(self, repository_info, tool):
-        """Returns the Review Board server url."""
+        """Return the Review Board server url.
+
+        Args:
+            repository_info (rbtools.clients.RepositoryInfo, optional):
+                Information about the current repository
+
+            tool (rbtools.clients.SCMClient, optional):
+                The repository client.
+
+        Returns:
+            unicode:
+            The server URL.
+        """
         if self.options.server:
             server_url = self.options.server
-        else:
+        elif tool:
             server_url = tool.scan_for_server(repository_info)
+        else:
+            server_url = None
 
         if not server_url:
             print('Unable to find a Review Board server for this source code '
@@ -693,6 +746,23 @@ class Command(object):
                 die('HTTP authentication is required, but cannot be '
                     'used with --diff-filename=-')
 
+            # Interactive prompts don't work correctly when input doesn't come
+            # from a terminal. This could seem to be a rare case not worth
+            # worrying about, but this is what happens when using native
+            # Python in Cygwin terminal emulator under Windows and it's very
+            # puzzling to the users, especially because stderr is also _not_
+            # flushed automatically in this case, so the program just appears
+            # to hang.
+            if not sys.stdin.isatty():
+                logging.error('Authentication is required but input is not a '
+                              'tty.')
+                if sys.platform == 'win32':
+                    logging.info('Check that you are not running this script '
+                                 'from a Cygwin terminal emulator (or use '
+                                 'Cygwin Python to run it).')
+
+                raise CommandError('Unable to log in to Review Board.')
+
             print()
             print('Please log in to the Review Board server at %s.' %
                   urlparse(uri)[1])
